@@ -30,13 +30,13 @@ def setup():
     global old_config
 
     try:
-        from django.test.runner import DiscoverRunner
-        test_runner = DiscoverRunner()
+        from django.test.simple import DjangoTestSuiteRunner as TestSuiteRunner
     except ImportError:
-        # Django 1.5 did not have the DiscoverRunner
-        from django.test.simple import DjangoTestSuiteRunner
-        test_runner = DjangoTestSuiteRunner()
+        # DjangoTestSuiteRunner was deprecated in django 1.8:
+        # https://docs.djangoproject.com/en/1.8/internals/deprecation/#deprecation-removed-in-1-8
+        from django.test.runner import DiscoverRunner as TestSuiteRunner
 
+    test_runner = TestSuiteRunner()
     test_runner.setup_test_environment()
     old_config = test_runner.setup_databases()
 
@@ -45,4 +45,3 @@ def teardown():
     test_runner.teardown_databases(old_config)
     test_runner.teardown_test_environment()
 
-
